/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////
//
// CreateAccount.java -- This program creates a customer account in the
//    Customer data file
//
/////////////////////////////////////////////////////////////////////////
package customer;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.net.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.as400.access.*;


public class CreateAccount extends HttpServlet
{
  private static boolean debug = true;

  String sysName;
  String userID;
  String pw;

  // Define the AS400 system and connection
  AS400 sys;
  Connection connection = null;


    public void init(ServletConfig config)
    {
       boolean rc;

       try
       {
         super.init(config);

         // Register the JDBC driver
         try {
           DriverManager.registerDriver (new com.ibm.as400.access.AS400JDBCDriver());
         }
         catch(Exception e)
         {                                                                                          System.out.println("JDBC Driver not found");
         }
       }
       catch(Exception e)
       {
          e.printStackTrace();
       }

       rc = setup();
       if (!rc) {
          System.out.println("ERROR in reading properties file during setup.");
          // System.exit(0);
       }

       try {
         // Signon to the AS/400
         sys = new AS400(sysName, userID, pw);
         sys.setGuiAvailable(false);
       }
       catch(Exception e)
       {
          e.printStackTrace();
       }

       try {
          connection = DriverManager.getConnection( "jdbc:as400://" +
                                                    sysName + "/" +
                                                    "MYPOP", userID, pw );
       }
       catch(Exception e)
       {
          System.out.println("Error Setting the JDBC connection");
       }

    }

    public void doPost (HttpServletRequest req, HttpServletResponse res)
     throws ServletException, IOException
    {
        HttpSession s = req.getSession(true); // Set up a Session

        String collectionName = "MYPOP";
        String tableName = "Customer";
        String firstname   = req.getParameter("firstname");
        String lastname = req.getParameter("lastname");
        String address   = req.getParameter("address");
        String city = req.getParameter("city");
        String state   = req.getParameter("state");
        String zip = req.getParameter("zip");
        String country = req.getParameter("country");
        String password= req.getParameter("password");
        String errorText= "";


        if (debug)
        {
           System.out.println("firstname:   " + firstname);
           System.out.println("lastname: " + lastname);
           System.out.println("address:   " + address);
           System.out.println("city: " + city);
           System.out.println("state:   " + state);
           System.out.println("zip: " + zip);
           System.out.println("country:   " + country);
           System.out.println("password:   " + password);
         }

        String urlStr  = new String(HttpUtils.getRequestURL(req));
        String urlBase = urlStr.substring(0, urlStr.lastIndexOf("/") + 1);
        System.out.println(urlBase);

        ServletOutputStream out = res.getOutputStream();

        res.setContentType("text/html");

        // Check all parameters for valid values
        if (lastname.length() == 0)
           errorText += "Last name not entered.  " ;
        if (firstname.length() == 0)
           errorText += "First name not entered.  " ;
        if (address.length() == 0)
           errorText += "Address not entered.  ";
        if (city.length() == 0)
           errorText += "City not entered.  " ;
        if (state.length() == 0)
           errorText += "State not entered.  " ;
        if (country.length() == 0)
           errorText += "Country not entered.  ";
        if (password.length() == 0)
           errorText += "Password not entered.  " ;


        if (errorText.length() == 0)
        {
           String urlBase2 = "http://" + req.getServerName() + ":"
                                + Integer.toString(req.getServerPort())
                                + "/mypop/";

           URL url = new URL(urlBase2 + "thanks.html");
           URLConnection c = url.openConnection();


           try {

              // Make a userspace read call to the Mypop user space that
              // holds the value of the current customer record.
              // Get the value, add 100 to it, use this as the
              // customerID and store the new value back.

              // Initialize the user space
              UserSpace custIDSpc = new UserSpace(sys,"/QSYS.LIB/MYPOP.LIB/CUSTID.USRSPC");

              // Read entry from the userspace
              byte[] byteData = new byte[10];
              custIDSpc.read(byteData,0);
              AS400Bin4 bin4cvt = new AS400Bin4();
              Integer custIDInt = (Integer) bin4cvt.toObject(byteData,0);
              int custID = custIDInt.intValue();
              System.out.println("custID = " + custID);

              // Update the customer ID number
              custID = custID + 100;

              // Write the new value back to the user space
              Integer custIDInt2 = new Integer(custID);
              byte[] writeData = bin4cvt.toBytes(custIDInt2);
              custIDSpc.write(writeData, 0);

              //  Insert the new customer info into the database
              System.out.println("Create Statement");
              Statement updateTable = connection.createStatement();
              String lastStr = "'" + lastname + "'";
              String firstStr = "'" + firstname + "'";
              String addStr = "'" + address + "'";
              String cityStr = "'" + city + "'";
              String stateStr = "'" + state + "'";
              String zipStr = "'" + zip + "'";
              String countryStr = "'" + country + "'";
              String passwordStr = "'" + password + "'";
              System.out.println("Creating Query String");
	           //  Create query as string
              updateTable.executeUpdate("INSERT INTO " + tableName +
                                        " VALUES (" + custID + ", " + lastStr + ", " +
                                        firstStr + ", " + addStr + ", " + cityStr + ", " +
                                        stateStr + ", " + zipStr + ", " +
                                        countryStr + "," + passwordStr + ", 0) ");

              // Print a thanks to the customer
              out.println("<HTML>");
              out.println("<BODY BGCOLOR=\"blanchedalmond\">");
              out.println("<TITLE>Thanks!</TITLE>");
              out.println("<BR><FONT size=4>Thank you for creating an account with My Pop's!");
              out.println("<BR><BR>Your customer ID is <FONT color=\"red\"> " +
                     custID + ".</FONT> Use this ID to access My Pop's catalog in the future.</FONT>");
              out.println("<FORM ACTION=\"/servlet/customer.ShowCatalog\" METHOD=POST>");

	           out.println("<INPUT TYPE=hidden NAME=custID VALUE=" + custID + ">");
              out.println("<INPUT TYPE=submit NAME=goCatalog VALUE=\"View Product Catalog\">");
              out.println("</FORM></HTML>");
              System.out.println("Closing the statement");
              // Close the statement
              updateTable.close();

           }
           catch (Exception e) {
               System.out.println("ERROR: " + e.getMessage() + e);
           }

       }
       else
       {
          out.println(
              "<HTML> "+
              "<TITLE> Shop at My Pop's</TITLE> "+
              "<BODY BGCOLOR=\"blanchedalmond\" TEXT=\"black\"> " );

           out.println(
             "<H2>Create Account Failed - <BR>" +
               errorText + "<BR> Please Try Again... </H2></FONT></HTML> ");


           String urlBase2 = "http://" + req.getServerName() + ":"
                                + Integer.toString(req.getServerPort())
                                + "/mypop/";

           URL url = new URL(urlBase2 + "purchase02.html");
           URLConnection c = url.openConnection();
           InputStream in = c.getInputStream();

           byte[] data = new byte[4096];
           int bytesRead = 0;

           while (bytesRead != -1)
           {
               bytesRead = in.read(data);
               if (bytesRead > 0)
               {
                   out.write(data, 0, bytesRead);
               }
           }
       }

        // Close the writer
        out.close();

    }


    public void destroy(ServletConfig config)
    {
       try {
          if (connection != null)
             connection.close();
       }
       catch (Exception e) {
         e.printStackTrace();
       }
   }


   private boolean setup()
   {
       try
       {
          ResourceBundle rb = ResourceBundle.getBundle("customer.customer");


          String system = rb.getString("SYSTEM");

          if (system != null)
          {
             sysName = system;
          }
          else
          {
             System.out.println("Failed, could not get system name out of bundle");
             return false;
          }
          String password = rb.getString("CUSTOMERPWD");

          if (password != null)
          {
             pw = password;
          }
          else
          {
             System.out.println("Failed, could not get password out of bundle");
             return false;
          }

          userID = "CUSTOMER";

          return true;

       }
       catch (Exception e)
       {
          System.out.println("Failed, could not load resources");
          e.printStackTrace();
          return false;
       }
 }



    public String getServletInfo()
    {
        return "AS/400 Create Account Applet";
    }
}
