/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////
//
//  ShowOrder.java -- This program handles all of the order processing from
//      the customer perspective.
//
///////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////
// This package statement precedes all other import statements.
// This lets the application server know that servlet 'ShowOrder'
// is found under ~home/servlet/customer directory, and can be
// invoked with the call: ~/servlet/customer.ShowOrder
///////////////////////////////////////////////////////////////////////
package customer;



import java.io.*;
import java.sql.*;
import java.util.*;
import java.net.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.as400.access.*;


//////////////////////////////////////////////////////////////////////

public class ShowOrder extends HttpServlet
{

  private static UserSpace orderIDSpc;

  private static boolean debug = true;

  String sysName;
  String userID;
  String pw;

  // Define an AS400 system object
  AS400 sys;

  // Initialize data connection
  Connection connection = null;


  public int custID;
  public int catalogID;
  public double price;
  public int quantity;
  public String product;
  public String action;
  public int actionCode=0;
  public String quantityStr;
  public String priceStr;

/////////////////////////////////////////////////////////////

  public void init(ServletConfig config)
  {
    boolean rc;

    try
      {
	// This statement helps to save the current (this) servlet's
	// configuration info.
	super.init(config);


	System.out.println("init called");

      }
    catch(Exception e)
      {
	e.printStackTrace();
      }

    rc = setup();

    if (!rc) {
      System.out.println("ERROR in reading properties file during setup.");
      System.exit(0);
    }

    // Register the JDBC driver
    try {

       DriverManager.registerDriver(new com.ibm.as400.access.AS400JDBCDriver());

    }
    catch(Exception e)
    {  System.out.println("JDBC Driver not found");
    }

    try {

      // Signon to the AS/400
      sys = new AS400(sysName, userID, pw);

      // Setup processing for IFS file
      // ConnectService to IFS for writing to the orders log
      sys.connectService(AS400.FILE);

      // Connect to the MyPop collection.
       connection = DriverManager.getConnection("jdbc:as400://" +
               sysName + "/" + "MYPOP", userID, pw);
    }
    catch(Exception e)
    {
       e.printStackTrace();
    }


    try {
       // Initialize the user space
       orderIDSpc = new UserSpace(sys,"/QSYS.LIB/MYPOP.LIB/ORDERID.USRSPC");
    }
    catch(Exception e)
    {
       System.out.println("Could not create user space");
       e.printStackTrace();
    }



  }

/////////////////////////////////////////////////////////////////

  public void doPost (HttpServletRequest req, HttpServletResponse res)
       throws ServletException, IOException
  {

    // invoke the session
    HttpSession s = req.getSession(false);
    String SessionID = s.getId();
    System.out.println("sessionid:   " + SessionID);

    ServletOutputStream out = res.getOutputStream();

    // get custid from session
    String custIDStr = (String) s.getValue("custIDstr");
    custID = Integer.valueOf(custIDStr).intValue();

    // get catalogid from session
    String catalogIDStr   = (String) s.getValue("catalogIDstr");
    catalogID = Integer.valueOf(catalogIDStr).intValue();

    // get action and put it into session, or renew it
    action = req.getParameter("action");
    s.putValue("action",action);  // in case it has changed


    if (action.indexOf("ADD") != -1)  // if the action is 'ADD'
      {
	// get product and put it into session, if necessary
	product = req.getParameter("product");
	s.putValue("product",product);

	// get quantity and put it into session, if necessary
	quantityStr = req.getParameter("quantity");
	s.putValue("quantity",quantityStr);
	quantity = Integer.valueOf(quantityStr).intValue();

	// get price and put it into session, if necessary
	priceStr = req.getParameter("price");
	s.putValue("price",priceStr);
	price = Double.valueOf(priceStr).doubleValue();
      }

    boolean found=false;
    Vector orderVector, existingVector, removeVector;

    if (action.indexOf("ADD") != -1) actionCode = 1;
    else if (action.indexOf("DISPLAY") != -1) actionCode = 2;
    else if (action.indexOf("BUY") != -1) actionCode = 3;
    else if (action.indexOf("ERASE") != -1) actionCode=4;

    if (debug)
      {
	System.out.println("custID:   " + custID);
	System.out.println("product:   " + s.getValue("product"));
	System.out.println("quantity:   " + s.getValue("quantity"));
	System.out.println("price:   " + s.getValue("price"));
	System.out.println("action:   " + s.getValue("action"));
      }

    String urlStr  = new String(HttpUtils.getRequestURL(req));
    String urlBase = urlStr.substring(0, urlStr.lastIndexOf("/") + 1);
    System.out.println(urlBase);

    res.setContentType("text/html");

    String urlBase2 = "http://" + req.getServerName() + ":" +
                        Integer.toString(req.getServerPort()) + "/mypop/";

    // Check which action is to be performed
    switch (actionCode) {

    case 1:

      // Create new orderobject to add to hashtable
      OrderObject orderObject = new OrderObject(catalogID, product, quantity, price);

      // Retrieve the entry from the session
      existingVector = (Vector) s.getValue("OrderEntries");

      if (existingVector == null)  // No entry found so this is first add
      {
         orderVector = new Vector();

         // Add an entry to the orders vector
         orderVector.addElement(orderObject);

         // Add vector to session
         s.putValue("OrderEntries", orderVector);

      }
      else
      { // Vector found so either add a new element or update an existing

         // Enumerate over the vector
         for (Enumeration x=existingVector.elements(); x.hasMoreElements();)
         {
            OrderObject existingOrder = (OrderObject) x.nextElement();
            if (existingOrder.Product.equals(orderObject.Product)) {
               existingOrder.Quantity = orderObject.Quantity;
               found=true;
            }
         }

	      if (!found)
            existingVector.addElement(orderObject);


	      // Add vector to session
	      s.putValue("OrderEntries",existingVector);

      }

      // Call routine to display the orders for a custID
      displayOrders(s,out);

      break;

    case 3:

      // Get the entries out of the hashtable and create an order for each one
      enterOrders(s,out);

      break;

    case 4:

      // Remove the entries from the session

      removeVector = (Vector) s.getValue("OrderEntries");
      removeVector.removeAllElements();
      s.putValue("OrderEntries",removeVector);


      // Call routine to display the orders for a custID

      displayOrders(s,out);

      break;

    default:
      break;

    }

    // Close the writer
    out.close();

  }

/////////////////////////////////////////////////////////////

  public String getServletInfo()
  {
    return "AS/400 Show Catalog Servlet";
  }

////////////////////////////////////////////////////////////

  public void displayOrders(HttpSession s, ServletOutputStream outStream) throws IOException
  {
    double TotalOrders = 0;
    Vector enumVector;

    outStream.println(
		      "<HTML> "+
		      "<TITLE> Shop at My Pop's</TITLE> "+
		      "<META HTTP-EQUIV=\"pragma\" content=\"no-cache\"> " +
		      "<BODY BGCOLOR=\"blanchedalmond\" TEXT=\"black\"> ");

    if (actionCode==1)
      outStream.println("<P><FONT COLOR=\"mediumvioletred\"> " +
			"<H2>Item Added to Shopping Cart</H2></FONT><P><HR>" +
			"<FONT COLOR=\"mediumvioletred\">" );
    else if (actionCode==4)
      outStream.println("<P><FONT COLOR=\"mediumvioletred\"> " +
			"<H2>Shopping Cart Emptied</H2></FONT><P><HR>" +
			"<FONT COLOR=\"mediumvioletred\">" );

    outStream.println("<H2>Current Shopping List:</H2></FONT>" +
		      "<P><CENTER><TABLE BORDER><TR><TH>Product</TH>" +
		      "<TH>Quantity</TH><TH>Price</TH><TH>Total</TH>");

    // Enumerate over the vector
    enumVector = (Vector) s.getValue("OrderEntries");
    for (Enumeration v=enumVector.elements(); v.hasMoreElements();)
      {
	OrderObject order = (OrderObject) v.nextElement();

	outStream.println("<TR><TD>" + order.Product + "</TD><TD>" + order.Quantity + "</TD>" +
			  "<TD ALIGN=Right>" + order.Price + "</TD><TD ALIGN=Right>" + order.Price*order.Quantity + "</TD></TR>");

	// Keep running total
	TotalOrders = TotalOrders + order.Price*order.Quantity;
      }


    outStream.println("<TR><TD COLSPAN=3>Estimated Total:</TD>" +
		      "<TD ALIGN=right>" + TotalOrders + "</TD></TR></TABLE></CENTER><P>");

    outStream.println("<CENTER><TABLE><TR>");


    outStream.println("<FORM ACTION=\"/servlet/customer.ShowOrder\" METHOD=POST>");
    outStream.print("<INPUT TYPE=hidden name=action value=BUY>" + "<TD><INPUT TYPE=submit VALUE=\"Buy, Buy, Buy...\"></TD></FORM>" );


    outStream.println("<FORM ACTION=\"/servlet/customer.ShowOrder\" METHOD=POST>");
    outStream.print("<INPUT TYPE=hidden name=action value=ERASE>" + "<TD><INPUT TYPE=submit VALUE=\"Empty Shopping Cart\"></TD></FORM>" );
    outStream.print("</TR></TABLE></CENTER><BR>");


    outStream.println("<FORM ACTION=\"/servlet/customer.ShowCatalog\" METHOD=POST>");
    outStream.println("<INPUT TYPE=image ALIGN=\"center\" ALT=\"Continue Shopping\" SRC=\"/mypop/pics/shopcart.gif\">" +
		      "<FONT size=4> <strong>Continue Shopping...</strong></FORM>");


    outStream.println("</FONT></BODY></HTML>");

    outStream.close();

  }

  private boolean setup()
  {
    try
      {
	ResourceBundle rb = ResourceBundle.getBundle("customer.customer");

	String system = rb.getString("SYSTEM");

	if (system != null)
          {
	    sysName=system;
          }
	else
          {
	    System.out.println("Failed, could not get system name out of bundle");
	    return false;
          }
	String password = rb.getString("CUSTOMERPWD");

	if (password != null)
          {
	    pw =password;
          }
	else
          {
	    System.out.println("Failed, could not get password out of bundle");
	    return false;
          }

	userID = "CUSTOMER";

	return true;

      }
    catch (Exception e)
      {
	System.out.println("Failed, could not load resources");
	e.printStackTrace();
	return false;
      }
  }

////////////////////////////////////////////////////////////

  public void enterOrders(HttpSession s, ServletOutputStream outStream) throws IOException
  {
    Vector removeVector;
    Vector f = (Vector) s.getValue("OrderEntries");
    Enumeration e = f.elements();

    String OrderLogStr;
    byte[] data = new byte[200];


    if (e.hasMoreElements() == false) {

      // Output a message saying order not processed and return
      outStream.println("<HTML><TITLE> Shop at My Pop's</TITLE> "+
			"<META HTTP-EQUIV=\"pragma\" content=\"no-cache\"> " +
			"<BODY BGCOLOR=\"blanchedalmond\" TEXT=\"black\"> " +
			"<BR><FONT size=4>Your order cannot be processed because you " +
			"have not selected any products.  Please select Continue Shopping " +
			"to return to our product catalog.");
      outStream.println("<FORM ACTION=\"/servlet/customer.ShowCatalog\" METHOD=POST>");
      outStream.println("<INPUT TYPE=image ALIGN=\"center\" ALT=\"Continue Shopping\" SRC=\"/mypop/pics/shopcart.gif\">" + "<FONT size=4> <strong>Continue Shopping...</strong></FORM>");
      outStream.println("</FONT></BODY></HTML>");
      return;
    }


    double TotalOrders = 0;

    // Set up the file info
    String collectionName = "MYPOP";
    String tableName = "Orders";

    try {

      // Make a UserSpace read call to the orderID user space that
      // holds the value of the current order record.
      // Get the value, add 100 to it, use this as the
      // orderID and store the new value back.


      // Create a file object that
      // represents the directory.
      IFSFile MyPopDir = new IFSFile(sys, "/MyPop");


      // Create the MyPopDir directory, if it doesn't already exist
      if (MyPopDir.mkdir())
         System.out.println("Create directory was successful");
      // Else the create directory failed.
      else
      {
         // If the object already exists,
	      // find out if it is a directory or
	      // file, then display a message.
	     if (MyPopDir.exists())
	     {
	        if (MyPopDir.isDirectory())
              System.out.println("Directory already exists");
	        else
              System.out.println("File with this name already exists");
	     }
	     else
	      System.out.println("Create directory failed");
	   }

      // Open the OrderLog file
      IFSFileOutputStream OrderTarget = new IFSFileOutputStream(sys,
								"/MyPop/OrderLog", IFSFileOutputStream.SHARE_ALL, true);

      // Read entry from the userspace
      byte[] byteData = new byte[10];
      orderIDSpc.read(byteData,0);
      AS400Bin4 bin4cvt = new AS400Bin4();
      Integer orderIDInt = (Integer) bin4cvt.toObject(byteData,0);
      int orderID = orderIDInt.intValue();
      //System.out.println("orderID = " + orderID);

      // Update the order ID number
      orderID = orderID + 100;
      System.out.println("orderID = " + orderID);

      // Write the new value back to the userspace
      Integer orderIDInt2 = new Integer(orderID);
      byte[] writeData = bin4cvt.toBytes(orderIDInt2);
      orderIDSpc.write(writeData, 0);

      // Insert the new order info into the database
      Statement updateTable = connection.createStatement();
      java.util.Date today = new java.util.Date();
      Timestamp todayTS = new Timestamp(today.getTime());
      String TimeStampStr = "{ts '" + todayTS + "' }";

	   // Enumerate over the vector
	   Vector tempVector = (Vector) s.getValue("OrderEntries");
	   for (Enumeration v=tempVector.elements(); v.hasMoreElements();)
	   {
	     OrderObject order = (OrderObject) v.nextElement();

	     // Add order to order table on the AS/400
	     updateTable.executeUpdate("INSERT INTO " + tableName +
				      " VALUES (" + orderID + ", " + custID + ", " +
				      order.CatalogID + ", " + order.Quantity + ", " +
				      order.Quantity*order.Price + ", " +
				      TimeStampStr + ", " + null + " ) ");

	     // Add an entry to order log
	     OrderLogStr = "\n" + " \n" + "** CUSTOMER ORDER RECEIVED -- Date: " + today.toString() + "\n" +
	       "   Order info -- OrderID = " + orderID + " Customer ID = " + custID +
	       " Catalog ID = " + order.CatalogID + "\n   Price = " + order.Price + " Quantity = " + order.Quantity;
	     data = OrderLogStr.getBytes();
	     OrderTarget.write(data);
	     OrderTarget.flush();
	   }

      // Close the output stream
      OrderTarget.close();

      // Remove the entry from the hashtable
      removeVector = (Vector) s.getValue("OrderEntries");
      removeVector.removeAllElements();
      s.putValue("OrderEntries",removeVector);

      // Put an entry on the WAREHOUSE dataqueue
      // Initialize the data queue
      KeyedDataQueue wdq = new KeyedDataQueue(sys,"/QSYS.LIB/MYPOP.LIB/WAREHOUSE.DTAQ");

      Integer custIDIntW = new Integer(orderID);
      byte[] writeData2 = bin4cvt.toBytes(custIDIntW);
      wdq.write(writeData2,writeData2);

      // Close the statement
      updateTable.close();

      // Output a thank-you
      outStream.println(
			"<HTML> "+
			"<TITLE> Shop at My Pop's</TITLE> "+
			"<META HTTP-EQUIV=\"pragma\" content=\"no-cache\"> " +
			"<BODY BGCOLOR=\"blanchedalmond\" TEXT=\"black\"> " +
			"<BR><FONT size=4>Your order has been sent. Your Order No: " + orderID + "<BR>");
      outStream.println("<BR>Thanks for shopping at My Pop's!</FONT></BODY></HTML>");
      outStream.println("<FORM ACTION=\"/servlet/customer.ShowCatalog\" METHOD=POST>");
      outStream.println("<INPUT TYPE=image ALIGN=\"center\" ALT=\"Continue Shopping\" SRC=\"/mypop/pics/shopcart.gif\">" +
			"<FONT size=4> <strong>Continue Shopping...</strong></FORM>");
      outStream.println("</FONT></BODY></HTML>");

    }
    catch (Exception ex) {
      ex.printStackTrace();
      outStream.println("ERROR: " + ex.getMessage());
      System.out.println("ERROR: " + ex.getMessage());
    }
  }
}
