/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////
//
//  SignIn.java -- This program retrieves the customer record
//
///////////////////////////////////////////////////////////////////////
package customer;

import java.io.*;
import java.sql.*;
import java.util.*;
import java.net.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.ibm.as400.access.*;

public class SignIn extends HttpServlet
{

  private static boolean debug = true;

  String sysName;
  String userID;
  String pw;


  // Define data connection
  Connection connection = null;


  public void init(ServletConfig config)
  {
    boolean rc;

    try {
      super.init(config);

      // Register the JDBC driver
      try {                                                                                   DriverManager.registerDriver(new com.ibm.as400.access.AS400JDBCDriver());
      }
      catch(Exception e)
      {
        System.out.println("JDBC Driver not found");
      }

    }
    catch(Exception e)
    {
       e.printStackTrace();
    }

    rc = setup();
    if (!rc)
    {
       System.out.println("ERROR in reading properties file during setup.");
       // System.exit(0);
    }

    try {

       connection = DriverManager.getConnection( "jdbc:as400://" +
                                                 sysName + "/" +
                                                 "MYPOP", userID, pw );

	 }
    catch(Exception e)
	 {
       System.out.println( "Error getting JDBC Connection." );
	 }

    }

    public void doPost (HttpServletRequest req, HttpServletResponse res)
     throws ServletException, IOException
    {

	     HttpSession s = req.getSession(true); // Set up a session


        String collectionName = "MYPOP";
        String tableName = "Customer";
        String custIDStr   = req.getParameter("custID");
        String password = req.getParameter("password");
        String errorText= "";



        if (debug)
        {
           System.out.println("customerID:   " + custIDStr);
           System.out.println("password: " + password);
        }

        String urlStr  = new String(HttpUtils.getRequestURL(req));
        String urlBase = urlStr.substring(0, urlStr.lastIndexOf("/") + 1);
        System.out.println(urlBase);

        ServletOutputStream out = res.getOutputStream();

        res.setContentType("text/html");

        // Check all parameters for valid values
        if (custIDStr.length() == 0)
           errorText += "Customer ID not entered.  ";
        if (password.length() == 0)
           errorText += "Password not entered.  " ;

        if (errorText.length() == 0)
        {
           // Create integer representation of customer ID
           int custID = Integer.valueOf(custIDStr).intValue();

           try {

	         // Insert the new customer info into the database
              Statement qryTable = connection.createStatement();
              String passwordStr = "'" + password + "'";

	           ResultSet rs = qryTable.executeQuery ("SELECT * FROM " + tableName +
                                                    " WHERE CUSTOMERID = " + custID +
                                                    " AND PW = " + passwordStr);


              if (rs.next()==false) {
                 // Output a message to the customer saying customer ID
                 // not found. Please try again
                 out.println ("<HTML> " +
                              "<TITLE> Shop at My Pop's</TITLE> " +
                              "<META HTTP-EQUIV=\"pragma\" content=\"no-cache\"> " +
                              "<BODY BGCOLOR=\"blanchedalmond\" TEXT=\"black\"> " );
                 out.println ("<FONT SIZE=4 COLOR=red>Customer record not found.  Please Try Again.<BR></FONT> ");
                 out.println ("<FONT SIZE=4><BR>If you continue to experience problems, please " +
                              "create a new account.</FONT>");
              }
              else {
                 // Print a thanks to the customer
                 String lastname = rs.getString("LASTNAME");
                 String firstname = rs.getString("FIRSTNAME");
                 out.println ("<HTML>");
                 out.println ("<TITLE>Welcome!</TITLE>");
                 out.println ("<META HTTP-EQUIV=\"pragma\" content=\"no-cache\"> ");
                 out.println ("<BODY BGCOLOR=\"blanchedalmond\">");
                 out.println ("<BR>Welcome to My Pop's, " + firstname + " " + lastname + "!");
                 out.println ("<FORM ACTION=\"/servlet/customer.ShowCatalog\" METHOD=POST>");
                 out.println ("<INPUT TYPE=hidden NAME=custID VALUE=" + custID + ">");
                 out.println ("<INPUT TYPE=submit NAME=goCatalog VALUE=\"View Product Catalog\">");
                 out.println ("</FORM></HTML>");
              }


              // Close the statement
              qryTable.close();
	        }
           catch (Exception e) {
              System.out.println("ERROR: " + e.getMessage());
           }
        }
        else
        {
           // Output a message to the customer saying customer ID not found. Please
           // try again
           out.println ("<HTML> " +
                        "<TITLE> Shop at My Pop's</TITLE> " +
                        "<META HTTP-EQUIV=\"pragma\" content=\"no-cache\"> " +
                        "<BODY BGCOLOR=\"blanchedalmond\" TEXT=\"black\"> " );

           out.println ("<H2>ERROR in customer data - <BR>" +
                        errorText + "<BR> Please Try Again... </H2></FONT></HTML> ");
        }
	     // Close the writer
	     out.close();

    }




    public void destroy(ServletConfig config)
    {
       try {
          if (connection != null) {
             connection.close();
          }
       }
       catch (Exception e) {
         e.printStackTrace();
       }
   }

   private boolean setup()
   {
       try
       {
          ResourceBundle rb = ResourceBundle.getBundle("customer.customer");

          String system = rb.getString("SYSTEM");

          if (system != null)
          {
             sysName=system;
          }
          else
          {
             System.out.println("Failed, could not get system name out of bundle");
             return false;
          }
          String password = rb.getString("CUSTOMERPWD");

          if (password != null)
          {
             pw =password;
          }
          else
          {
             System.out.println("Failed, could not get password out of bundle");
             return false;
          }

          userID = "CUSTOMER";

          return true;

       }
       catch (Exception e)
       {
          System.out.println("Failed, could not load resources");
          e.printStackTrace();
          return false;
       }
 }

    public String getServletInfo()
    {
        return "AS/400 SignIn Servlet";
    }
}
