/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import com.ibm.as400.vaccess.*;
import java.sql.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class CatalogDB extends JComponent
{
    boolean isJComponent_ = true;

    SQLResultSetTablePane tablePane;
    JFrame frame_;
    AS400  system;
    String user;
    String password;
    String collectionName = "MYPOP";

    public CatalogDB(AS400 sys, boolean isJComponent)
    {
        super();

        system = sys;

        isJComponent_ = isJComponent;
    }


    public static void main (String[] args)
    {
        // If a system was not specified, then display help text and
        // exit.
        if (args.length == 0)
        {
            System.out.println("Usage: java CatalogDB <system> <userid> <password>");
            return;
        }

        try
        {
           AS400 sys = new AS400(args[0]);

           if (args.length > 1)
              sys.setUserId(args[1]);

           if (args.length > 2)
              sys.setPassword(args[2]);

           if (args.length > 2)
              sys.addPasswordCacheEntry(args[0], args[1], args[2]);

           CatalogDB catalogDB = new CatalogDB(sys, false);

           catalogDB.run();
        }
        catch (Exception e) { e.printStackTrace(); }
    }



    public void setFrame(JFrame frame)
    {
        frame_ = frame;
    }


    public void run()
    {
        try
        {
            // Force the user to sign on so that we know the user id.
            DriverManager.registerDriver(new AS400JDBCDriver());
            SQLConnection c = new SQLConnection("jdbc:as400://" + system.getSystemName() + "/" + collectionName, system.getUserId() );

            if (!isJComponent_)
            {
                // Create a frame.
                frame_ = new JFrame ("JDBC Database Example");

                // When the frame closes, exit.
                frame_.addWindowListener (new WindowAdapter ()
                {
                    public void windowClosing(WindowEvent e)
                    {
                        frame_.dispose();
                        if (!isJComponent_) {
                            System.exit(0);
                        }
                    }
                } );
            }

            // Create an error dialog adapter.  This will display
            // any errors to the user.
            ErrorDialogAdapter errorHandler = new ErrorDialogAdapter (frame_);

            tablePane = new SQLResultSetTablePane (c, "SELECT * FROM MYPOP.PRODCAT");

            tablePane.load();

            alterColumns();

            if (!isJComponent_)
            {
                // Layout the frame with the tree pane.
                frame_.getContentPane().setLayout (new BorderLayout ());
                frame_.getContentPane().add ("North", tablePane);
            }
            else {
                setLayout(new BorderLayout());
                add("Center", tablePane);
            }

            // Create a button for displaying results in a table.
            JButton refreshButton = new JButton("Refresh");
            refreshButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   tablePane.load();
                   alterColumns();
                }
            } );

            // Add the buttons to the GUI.
            JPanel buttonPanel = new JPanel();
            buttonPanel.add(refreshButton);

            if (!isJComponent_) {
                frame_.getContentPane().add("South", buttonPanel);
                frame_.pack();
                frame_.setSize(550,750);
                frame_.setResizable(true);
                frame_.setVisible(true);
                frame_.pack ();
                frame_.show ();
            }
            else
            {
                add("South", buttonPanel);
            }
        }
        catch (Exception e)
        {
            System.out.println ("Error in Product Database");
            e.printStackTrace();
            if (!isJComponent_) {
                System.exit (0);
            }
        }
    }


    void alterColumns()
    {
       tablePane.setColumnTitle(0, "Catalog ID");
       tablePane.setColumnTitle(1, "Retail Price");
       tablePane.setColumnTitle(2, "Units Available");
       tablePane.setColumnTitle(3, "Description");

       tablePane.setColumnWidth(0, 100);
       tablePane.setColumnWidth(1, 100);
       tablePane.setColumnWidth(2, 100);
    }
}
