/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import com.ibm.as400.vaccess.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class CustomerDB extends JComponent
{
    boolean isJComponent_ = true;

    AS400 system;
    RecordListTablePane tablePane;
    JFrame frame_;

    public CustomerDB(AS400 sys, boolean isJComponent)
    {
        super();

        system = sys;

        isJComponent_ = isJComponent;

        if (isJComponent_)
           run();
    }


    public static void main (String[] args)
    {
        // If a system was not specified, then display help text and
        // exit.
        if (args.length == 0)
        {
            System.out.println("Usage: java CustomerDB <system>");
            return;
        }

        try
        {
           AS400 s = new AS400(args[0]);

           if (args.length > 1)
              s.setUserId(args[1]);

           if (args.length > 2)
              s.setPassword(args[2]);

           CustomerDB customerDB = new CustomerDB(s, false);

           customerDB.run();
        }
        catch (Exception e) { e.printStackTrace(); }
    }



    public void setFrame(JFrame frame)
    {
        frame_ = frame;
    }


    public void run()
    {
        try
        {
            // Force the user to sign on so that we know the user id.
            system.connectService(AS400.RECORDACCESS);

            if (!isJComponent_)
            {
                // Create a frame.
                frame_ = new JFrame ("RLA Database Example");

                // When the frame closes, exit.
                frame_.addWindowListener (new WindowAdapter ()
                {
                    public void windowClosing(WindowEvent e)
                    {
                        frame_.dispose();
                        if (!isJComponent_) {
                            System.exit(0);
                        }
                    }
                } );
            }

            // Create an error dialog adapter.  This will display
            // any errors to the user.
            ErrorDialogAdapter errorHandler = new ErrorDialogAdapter (frame_);

            tablePane = new RecordListTablePane (system, "/QSYS.LIB/MYPOP.LIB/CUSTOMER.FILE");

            tablePane.load();

            alterColumns();

            if (!isJComponent_)
            {
                // Layout the frame with the tree pane.
                frame_.getContentPane().setLayout (new BorderLayout ());
                frame_.getContentPane().add ("North", tablePane);
            }
            else {
                setLayout(new BorderLayout());
                add("Center", tablePane);
            }

            // Create a button for displaying results in a table.
            JButton refreshButton = new JButton("Refresh");
            refreshButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   System.out.println("refresh pressed");
                   tablePane.load();
                   alterColumns();
                }
            } );

            // Add the buttons to the GUI.
            JPanel buttonPanel = new JPanel();
            buttonPanel.add(refreshButton);

            if (!isJComponent_) {
                frame_.getContentPane().add("South", buttonPanel);
                frame_.pack();
                frame_.setSize(550,750);
                frame_.setResizable(true);
                frame_.setVisible(true);
                frame_.pack ();
                frame_.show ();
            }
            else
            {
                add("South", buttonPanel);
            }
        }
        catch (Exception e)
        {
            System.out.println ("Error in Customer Database");
            e.printStackTrace();
            if (!isJComponent_) {
                System.exit (0);
            }
        }
    }

    void alterColumns()
    {

       tablePane.setColumnTitle(0, "Customer ID");
       tablePane.setColumnTitle(1, "Last Name");
       tablePane.setColumnTitle(2, "First Name");
       tablePane.setColumnTitle(3, "Street");
       tablePane.setColumnTitle(4, "City");
       tablePane.setColumnTitle(5, "State");
       tablePane.setColumnTitle(6, "ZIP");
       tablePane.setColumnTitle(7, "Country");


       DefaultTableColumnModel tcm = (DefaultTableColumnModel) tablePane.getColumnModel();
       tcm.removeColumn(tcm.getColumn(8));

       DefaultTableColumnModel tcm2 = (DefaultTableColumnModel) tablePane.getColumnModel();
       tcm2.removeColumn(tcm2.getColumn(8));

       tablePane.setColumnWidth(0, 100);
       tablePane.setColumnWidth(1, 100);
       tablePane.setColumnWidth(2, 100);
       tablePane.setColumnWidth(3, 150);
       tablePane.setColumnWidth(4, 100);
       tablePane.setColumnWidth(6, 100);
       tablePane.setColumnWidth(7, 100);
    }
}
