/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import com.ibm.as400.vaccess.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class HistoryLog extends JComponent
{
    boolean isJComponent_ = true;

    AS400 system;

    JFrame frame_;

    static IFSTextFileDocument document = null;
    JTextPane textPane;

    static final int WIDTH  = 750;
    static final int HEIGHT = 550;

    public HistoryLog(AS400 sys, boolean isJComponent)
    {
        super();

        system = sys;

        isJComponent_ = isJComponent;

        if (isJComponent_)
           run();
    }



    public static void main (String[] args)
    {
        // If a system was not specified, then display help text and
        // exit.
        if (args.length == 0)
        {
            System.out.println("Usage: java HistoryLog <system>");
            return;
        }

        try
        {
           AS400 s = new AS400(args[0]);

           if (args.length > 1)
              s.setUserId(args[1]);

           if (args.length > 2)
              s.setPassword(args[2]);

           HistoryLog historyLog = new HistoryLog(s, false);

           historyLog.run();
        }
        catch (Exception e) { e.printStackTrace(); }
    }



    public void setFrame(JFrame frame)
    {
        frame_ = frame;
    }


    public void run()
    {
        try
        {
            // Create a VMessageQueue object which represents the
            // current user's message queue.
            if (document == null)
               document = new IFSTextFileDocument(system, "/mypop/OrderLog");

            if (!isJComponent_)
            {
                // Create a frame.
                frame_ = new JFrame ("Text Document Example");


                Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
                frame_.setLocation(screenSize.width/2 - WIDTH/2,
                                   screenSize.height/2 - HEIGHT/2);

                frame_.setSize(WIDTH, HEIGHT);


                // When the frame closes, exit.
                frame_.addWindowListener (new WindowAdapter ()
                {
                    public void windowClosing(WindowEvent e)
                    {
                        frame_.dispose();
                        if (!isJComponent_) {
                            System.exit(0);
                        }
                    }
                } );
            }

            // Create an error dialog adapter.  This will display
            // any errors to the user.
            ErrorDialogAdapter errorHandler = new ErrorDialogAdapter (frame_);

            // Create an explorer pane to present the message queue.
            // Use load to load the information from the system.
            document.addErrorListener(errorHandler);
            document.load ();

            textPane = new JTextPane(document);
            textPane.setEditable(false);

            JScrollPane scroll = new JScrollPane(textPane);
            scroll.setHorizontalScrollBarPolicy (JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
            scroll.setVerticalScrollBarPolicy (JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);

            if (!isJComponent_)
            {
                // Layout the frame with the tree pane.
                frame_.getContentPane().setLayout (new BorderLayout ());
                frame_.getContentPane().add ("Center", scroll);
            }
            else {
                setLayout(new BorderLayout());
                add("Center", scroll);
            }

            // Create a button for displaying results in a table.
            JButton refreshButton = new JButton("Refresh");
            refreshButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   System.out.println("HistoryLog.refreshButton (refresh pressed)");
                   refreshDocument();
                }
            } );

            // Add the buttons to the GUI.
            JPanel buttonPanel = new JPanel();
            buttonPanel.add(refreshButton);

            if (!isJComponent_) {
                frame_.getContentPane().add("South", buttonPanel);
                frame_.setResizable(true);
                frame_.setVisible(true);
                frame_.show ();
            }
            else
            {
                add("South", buttonPanel);
            }
        }
        catch (Exception e)
        {
            System.out.println ("Error in History Log");
            e.printStackTrace();
            if (!isJComponent_) {
                System.exit (0);
            }
        }
    }

    static public void refreshDocument()
    {
       if (document != null)
          document.load();
       else
          System.out.println("HistoryLog.refreshDocument (update failed, document is null) ");
    }

}
