/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import com.ibm.as400.vaccess.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class MsgQueue extends JComponent
{

    String label_  = null;
    String toUser_ = null;

    boolean isJComponent_ = true;

    AS400 system;

    JFrame frame_;

    AS400DetailsPane detailsPane;


    public MsgQueue(AS400 s, String label, String toUser, boolean isJComponent)
    {
        super();

        system = s;

        label_  = label;
        toUser_ = toUser;

        isJComponent_ = isJComponent;

        if (isJComponent_)
            run();
    }


    public static void main (String[] args)
    {
        // If a system was not specified, then display help text and
        // exit.
        if (args.length == 0)
        {
            System.out.println("Usage: java MsgQueue <system>");
            return;
        }

        try
        {
           AS400 s = new AS400 (args[0]);

           String strLabel  = null;
           String strToUser = null;

           if (args.length > 1)
              s.setUserId(args[1]);

           if (args.length > 2)
              s.setPassword(args[2]);

           if (args.length > 3)
              strLabel = args[3];

           if (args.length > 4)
              strToUser = args[4];

           MsgQueue messageQueue = new MsgQueue(s, strLabel, strToUser, false);

           messageQueue.run();
        }
        catch (Exception e)
        {
           System.out.println("Failed in MsgQueue (main)");
           e.printStackTrace();
        }
    }



    public void setFrame(JFrame frame)
    {
        frame_ = frame;
    }


    public void run()
    {
        try
        {
            // Force the user to sign on so that we know the user id.
            system.connectService (AS400.COMMAND);

            String queuePath = QSYSObjectPathName.toPath ("QUSRSYS", system.getUserId(), "MSGQ");

            // Create a VMessageQueue object which represents the
            // current user's message queue.
            VMessageQueue queue = new VMessageQueue (system, queuePath);

            if (!isJComponent_)
            {
                // Create a frame.
                frame_ = new JFrame ("Message queue example");

                // When the frame closes, exit.
                frame_.addWindowListener (new WindowAdapter ()
                {
                    public void windowClosing(WindowEvent e)
                    {
                        frame_.dispose();
                        if (!isJComponent_) {
                            System.exit(0);
                        }
                    }
                } );
            }

            // Create an error dialog adapter.  This will display
            // any errors to the user.
            ErrorDialogAdapter errorHandler = new ErrorDialogAdapter (frame_);

            // Create an explorer pane to present the message queue.
            // Use load to load the information from the system.
            detailsPane = new AS400DetailsPane(queue);
            detailsPane.addErrorListener(errorHandler);
            detailsPane.load ();

            if (!isJComponent_)
            {
                // Layout the frame with the tree pane.
                frame_.getContentPane().setLayout (new BorderLayout ());
                frame_.getContentPane().add ("North", detailsPane);
            }
            else {
                setLayout(new BorderLayout());
                add("Center", detailsPane);
            }

            // Create a button for displaying results in a table.
            JButton refreshButton = new JButton("Refresh");
            refreshButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   // System.out.println("refresh pressed");
                   detailsPane.load();
                }
            } );

            // Create a button for displaying results in a form.
            JButton sendButton = new JButton("Send a Message");
            sendButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   // System.out.println("send pressed");

                   SendMessage m = new SendMessage(system, label_, toUser_, false);
                   m.run();
                }
            } );

            // Add the buttons to the GUI.
            JPanel buttonPanel = new JPanel();
            buttonPanel.add(refreshButton);
            buttonPanel.add(sendButton);

            if (!isJComponent_) {
                frame_.getContentPane().add("South", buttonPanel);
                frame_.pack();
                frame_.setSize(550,750);
                frame_.setResizable(true);
                frame_.setVisible(true);
                frame_.pack ();
                frame_.show ();
            }
            else
            {
                add("South", buttonPanel);
            }
        }
        catch (Exception e)
        {
            System.out.println ("Error: " + e.getMessage ());
            if (!isJComponent_) {
                System.exit (0);
            }
        }
    }
}
