/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import com.ibm.as400.vaccess.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.sql.*;
import java.util.*;

public class ProcessOrder extends JComponent
{
    boolean isJComponent_ = true;
    boolean debugOn = false;

    JFrame frame_;

    JTextPane textPane;
    ErrorDialogAdapter errorHandler;
    JLabel textLabel;
    JButton refreshButton;
    JButton printButton;
    JButton doneButton;
    JButton nextButton;
    JPanel buttonPanel;

    KeyedDataQueue dq;
    AS400Bin4 intConverter;

    boolean firstTime = true;
    int panelState = 0;

    byte[] zero = new byte[] {0,0,0,0};
    byte[] keyToCheck = zero;

    AS400 system;

    String collectionName = "MYPOP";
    String tableName = "Orders";
    SQLConnection connection;
       Connection c;
    SQLResultSetTablePane tablePane;

    PreparedStatement queryPS;
    PreparedStatement updatePS;

    ResourceBundle rb;

    public ProcessOrder(AS400 sys, boolean isJComponent)
    {
        super();

        system = sys;

        isJComponent_ = isJComponent;
    }


    public static void main (String[] args)
    {
        // If a system was not specified, then display help text and
        // exit.
        if (args.length == 0)
        {
            System.out.println ("Usage: java mypop.ProcessOrder " +
                                "<system> [ <userID> [ <password> ]]");
            return;
        }

        try
        {
           AS400 sys = new AS400(args[0]);

           if (args.length > 1)
              sys.setUserId(args[1]);

           if (args.length > 2)
              sys.setPassword(args[2]);

           if (args.length > 2)
              sys.addPasswordCacheEntry(args[0], args[1], args[2]);

           ProcessOrder processOrder = new ProcessOrder(sys, false);

           processOrder.run();
        }
        catch (Exception e) { System.out.println("ProcessOrder.main"); e.printStackTrace(); }
    }



    public void setFrame(JFrame frame)
    {
        frame_ = frame;
    }


    public void run()
    {
        try
        {
            dq = new KeyedDataQueue(system, "/QSYS.LIB/MYPOP.LIB/WAREHOUSE.DTAQ");

            intConverter = new AS400Bin4();

            system.connectService(AS400.DATABASE);

            // Register the JDBC driver
            DriverManager.registerDriver(new com.ibm.as400.access.AS400JDBCDriver());

            connection = new SQLConnection("jdbc:as400://" + system.getSystemName() + "/" + collectionName, system.getUserId() );
            c = connection.getConnection();

            tablePane = new SQLResultSetTablePane();
            tablePane.setConnection(connection);

            updatePS = c.prepareStatement("UPDATE PRODCAT SET QUANTITYAVAILABLE = ? WHERE CATALOGID = ?");
            queryPS  = c.prepareStatement("SELECT ORDERS.CATALOGID, ORDERS.QUANTITYORDERED, PRODCAT.QUANTITYAVAILABLE FROM ORDERS, PRODCAT WHERE (PRODCAT.CATALOGID = ORDERS.CATALOGID) AND  (ORDERID = ?)");

            rb = ResourceBundle.getBundle("mypop.mypop");

            if (!isJComponent_)
            {
               // Create a frame.
               frame_ = new JFrame ("Process an Order");
               frame_.setSize(750, 550);
               frame_.setVisible(true);
               frame_.show();
               // frame_.setContentPane(this);

               // When the frame closes, exit.
               frame_.addWindowListener (new WindowAdapter ()
               {
                   public void windowClosing(WindowEvent e)
                   {
                       frame_.dispose();

                       if (!isJComponent_)
                       {
                           System.exit(0);
                       }
                   }
               } );
            }

            setLayout(new BorderLayout ());

            textLabel = new JLabel(" ", JLabel.CENTER);
            textLabel.setHorizontalTextPosition(JLabel.CENTER);
            textLabel.setVerticalTextPosition(JLabel.TOP);

            Font f = new Font(textLabel.getFont().getName(),
                              textLabel.getFont().getStyle(),
                              textLabel.getFont().getSize() + 4);

            textLabel.setFont(f);

            textLabel.setText("Temporary text for a Win 95 bug");

            // Create an error dialog adapter.  This will display
            // any errors to the user.
            errorHandler = new ErrorDialogAdapter (frame_);

            // Create a button for displaying results in a table.
            refreshButton = new JButton("Get Orders");
            refreshButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   panelProcessing();
                }
            } );

            // Add the buttons to the GUI.
            buttonPanel = new JPanel();
            buttonPanel.add(refreshButton);

            // Create a button for printing the mailing lable.
            printButton = new JButton("Print Mailing Label");
            printButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   Popup p = new Popup("zzzttt, zzzttt (sound of an impact printer)",
                                       " ",
                                       "Print Complete",
                                       false);
                   p.run();

                }
            } );

            // Create a button for indicating the order is done.
            doneButton = new JButton("Done With This Order");
            doneButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   try
                   {
                      if (removeQueueItem(keyToCheck))
                         dumpMessage("ProcessOrder.done (put to Q OK)", null);
                      else
                         dumpMessage("ProcessOrder.done (could not find Q entry)", null);
                   }
                   catch (Exception e) { dumpMessage("ProcessOrder.done (put to Q failed)", e); }


                   try
                   {
                      java.util.Date today = new java.util.Date();
                      String temp = "\n" + " \n" + "** ORDER PROCESSED -- Date: " + today.toString() + "\n" + "   OrderID = " + intConverter.toInt(keyToCheck);

                      IFSFileOutputStream log = new
                                   IFSFileOutputStream(system,
                                                       "/MyPop/OrderLog",
                                                       IFSFileOutputStream.SHARE_ALL,
                                                       true);
                      log.write(temp.getBytes());
                      log.flush();
                      log.close();
                   }
                   catch (Exception e) { dumpMessage("ProcessOrder.done (put to log failed)", e); }

                   HistoryLog.refreshDocument();

                   try
                   {
                      queryPS.setInt(1, intConverter.toInt(keyToCheck));

                      ResultSet rs = queryPS.executeQuery();

                      while (rs.next())
                      {
                         int amountLeft = rs.getInt("QUANTITYAVAILABLE") - rs.getInt("QUANTITYORDERED");
                         int catalogID  = rs.getInt("CATALOGID");

                         updatePS.setInt(1, amountLeft);
                         updatePS.setInt(2, catalogID);

                         updatePS.executeUpdate();
                      }
                   }
                   catch (Exception e) { dumpMessage("ProcessOrder.done (update units available failed)", e); }

                   panelProcessing();
                }
            } );

            // Create a button for moving on to the next item.
            nextButton = new JButton("Skip to Next Order");
            nextButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   panelProcessing();
                }
            } );

            // Add the buttons to the GUI.
            buttonPanel.add(printButton);
            buttonPanel.add(doneButton);
            buttonPanel.add(nextButton);

            if (!isJComponent_)
            {
               frame_.getContentPane().add("North", textLabel);
               frame_.getContentPane().add("Center", tablePane);
               frame_.getContentPane().add("South", buttonPanel);
            }
            else
            {
               add("North", textLabel);
               add("Center", tablePane);
               add("South", buttonPanel);
            }

            panelProcessing();
        }
        catch (Exception e)
        {
            dumpMessage("ProcessOrder.run", e);

            if (!isJComponent_)
            {
                System.exit (0);
            }
        }
    }


    void panelProcessing()
    {
        try
        {
            int orderID = checkForOrder(keyToCheck);

            if (orderID == 0)
            {
               panelState = 1;

               textLabel.setText("Nothing to do right now, Frank");

               tablePane.setQuery("SELECT ORDERS.CATALOGID, ORDERS.QUANTITYORDERED, PRODCAT.QUANTITYAVAILABLE, PRODCAT.DESCRIPTION FROM ORDERS, PRODCAT WHERE (PRODCAT.CATALOGID = ORDERS.CATALOGID) AND  (ORDERID = 0");

               tablePane.load();

               tablePane.setColumnTitle(0, "Catalog ID");
               tablePane.setColumnTitle(1, "Units Ordered");
               tablePane.setColumnTitle(2, "Units On Hand");
               tablePane.setColumnTitle(3, "Product Description");

               tablePane.setShowVerticalLines(false);

               tablePane.setColumnWidth(0, 100);
               tablePane.setColumnWidth(1, 100);
               tablePane.setColumnWidth(2, 100);

               keyToCheck = zero;

               refreshButton.setEnabled(true);
               printButton.setEnabled(false);
               doneButton.setEnabled(false);
               nextButton.setEnabled(false);


               if (!isJComponent_)
               {
                  frame_.show();
               }
            }
            else
            {

               textLabel.setText("Order Identifier: " + orderID );

               dumpMessage("ProcessOrder.panelProcessing (performing query)", null);

               tablePane.setQuery("SELECT ORDERS.CATALOGID, ORDERS.QUANTITYORDERED, PRODCAT.QUANTITYAVAILABLE, PRODCAT.DESCRIPTION FROM ORDERS, PRODCAT WHERE (PRODCAT.CATALOGID = ORDERS.CATALOGID) AND  (ORDERID = " + orderID + ")" );

               dumpMessage("ProcessOrder.panelProcessing (loading the table)", null);

               tablePane.load();


               tablePane.setColumnTitle(0, "Catalog ID");
               tablePane.setColumnTitle(1, "Units Ordered");
               tablePane.setColumnTitle(2, "Units On Hand");
               tablePane.setColumnTitle(3, "Product Description");

               tablePane.setShowVerticalLines(false);

               tablePane.setColumnWidth(0, 100);
               tablePane.setColumnWidth(1, 100);
               tablePane.setColumnWidth(2, 100);

               panelState = 2;

               keyToCheck = intConverter.toBytes(orderID);

               refreshButton.setEnabled(false);
               printButton.setEnabled(true);
               doneButton.setEnabled(true);
               nextButton.setEnabled(true);

               if (!isJComponent_)
               {
                  frame_.show();
               }
               else
               {
                  tablePane.repaint();
                  repaint();
                  show();
               }
            }
        }
        catch (Exception e)
        {
            dumpMessage("ProcessOrder.panelProcessing", e);

            if (!isJComponent_)
            {
                System.exit (0);
            }
        }
    }




    int checkForOrder(byte[] ID)
    {
        int orderID = 0;

        try
        {
           KeyedDataQueueEntry dqData = dq.peek(ID, 0, "GT");

           if (dqData != null)
           {
              orderID = intConverter.toInt(dqData.getKey(), 0);

              dumpMessage("ProcessOrder.checkForOrder, OrderID = " + orderID, null);
           }
           else
           {
              dumpMessage("ProcessOrder.checkForOrder (Q is empty)", null);

              DQAlert.queueEmpty();
           }
        }
        catch (Exception e) { dumpMessage("ProcessOrder.checkForOrder", e); }

        return orderID;
    }





    boolean removeQueueItem(byte[] ID)
    {
        boolean result = false;

        try
        {
           KeyedDataQueueEntry dqData = dq.read(ID);

           if (dqData != null)
           {
              result = true;
           }
        }
        catch (Exception e) { dumpMessage("ProcessOrder.removeQueueItem", e); }

        return result;
    }






    void dumpMessage(String message, Exception e)
    {
       if (debugOn)
       {
          System.out.println(message);

          if (e != null)
             e.printStackTrace();
       }
    }





}
