/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import java.sql.*;
import java.util.*;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;


public class ProcessOrderServlet extends HttpServlet
{
    private boolean debugOn = true;

    private AS400 system = null;
    private ShowTableServlet showTable = null;

    private KeyedDataQueue dq;
    private AS400Bin4 intConverter;
    private AS400Text textConverter4;

    private Connection connection;
    private PreparedStatement queryPS;
    private PreparedStatement queryPS2;
    private PreparedStatement updatePS;
    private String collectionName = "MYPOP";

    private int currentOrderID = 0;
    private int actionCode = INIT;


    private final static int INIT  = 0;
    private final static int RESET = 1;
    private final static int PRINT = 2;
    private final static int NEXT  = 3;
    private final static int DONE  = 4;
    private final static int NONE  = 5;






    public void doGet (HttpServletRequest req, HttpServletResponse res)
                       throws ServletException, IOException
    {
        showScreen(req, res);
    }




    public void doPost (HttpServletRequest req, HttpServletResponse res)
     throws ServletException, IOException
    {
        showScreen(req, res);
    }




    public void init(ServletConfig value)
                     throws ServletException
    {
        super.init(value);

        system = HarnessServlet.getSystem();

        if (system != null)
        {
           dq = new KeyedDataQueue(system, "/QSYS.LIB/MYPOP.LIB/WAREHOUSE.DTAQ");
           intConverter = new AS400Bin4();
           textConverter4 = new AS400Text(4);

           currentOrderID = checkForOrder(intConverter.toBytes(currentOrderID));


           try
           {
              DriverManager.registerDriver(new AS400JDBCDriver());
              connection = DriverManager.getConnection("jdbc:as400://" + system.getSystemName() + "/" + collectionName + "; user=" + system.getUserId());
              updatePS   = connection.prepareStatement("UPDATE PRODCAT SET QUANTITYAVAILABLE = ? WHERE CATALOGID = ?");
              queryPS    = connection.prepareStatement("SELECT ORDERS.CATALOGID, ORDERS.QUANTITYORDERED, PRODCAT.QUANTITYAVAILABLE FROM ORDERS, PRODCAT WHERE (PRODCAT.CATALOGID = ORDERS.CATALOGID) AND  (ORDERID = ?)");
              queryPS2   = connection.prepareStatement("SELECT CUSTOMERID FROM ORDERS WHERE ORDERID = ?");
           }
           catch (Exception e) { dumpDebugMessage("ProcesOrderServlet.setup (driver setup failed)", e); }


           showTable = new ShowTableServlet(system);
           showTable.setCollection("MYPOP");
           showTable.setQuery("SELECT ORDERS.CATALOGID, ORDERS.QUANTITYORDERED, PRODCAT.QUANTITYAVAILABLE, PRODCAT.DESCRIPTION FROM ORDERS, PRODCAT WHERE (PRODCAT.CATALOGID = ORDERS.CATALOGID) AND  (ORDERID = " + currentOrderID + ")" );

           showTable.setColumnName(1, "Catalog ID");
           showTable.setColumnName(2, "Unit Ordered");
           showTable.setColumnName(3, "Units Available");
           showTable.setColumnName(4, "Description");

           showTable.setShowHeader(false);

           showTable.setup();
        }
    }


    private void showScreen(HttpServletRequest req, HttpServletResponse res)
                            throws ServletException, IOException
    {
        ServletOutputStream out = res.getOutputStream();

        res.setContentType("text/html");

        String action = req.getParameter("action");

        boolean showButtons = true;

        if (action != null)
        {

           if (action.equals("Print Mailing Label"))
              actionCode = PRINT;

           else if (action.equals("Next Order"))
              actionCode = NEXT;

           else if (action.equals("Done With This Order"))
              actionCode = DONE;

           else if (action.equals("First Order"))
              actionCode = RESET;
        }

        out.println("<HTML> ");
        out.println("<META HTTP-EQUIV=pragma content=no-cache>");
        out.println("<BODY  BGCOLOR=blanchedalmond TEXT=black>");
        out.println("<TABLE BGCOLOR=blanchedalmond TEXT=black BORDER=1>");

        String extraMessage = "";

        if (system != null)
        {
           if (actionCode > INIT)
           {
              if (actionCode == PRINT)
              {
                 if (printMailingLabel(currentOrderID, out))
                    extraMessage = "<P><BR><P><CENTER><H3>Label Printed!!</H3></CENTER>";
                 else
                    extraMessage = "<P><BR><P><CENTER><H3>Print Failed!</H3></CENTER>";
              }
              else
              {
                 if (actionCode == RESET)
                    currentOrderID = 0;

                 if (actionCode == DONE)
                 {
                    finishOrder(currentOrderID);
                    extraMessage = "<P><BR><P>Order Processed</P>";
                 }

                 currentOrderID = checkForOrder(intConverter.toBytes(currentOrderID));
              }

              showTable.setQuery("SELECT ORDERS.CATALOGID, ORDERS.QUANTITYORDERED, PRODCAT.QUANTITYAVAILABLE, PRODCAT.DESCRIPTION FROM ORDERS, PRODCAT WHERE (PRODCAT.CATALOGID = ORDERS.CATALOGID) AND  (ORDERID = " + currentOrderID + ")" );
           }

           if (currentOrderID > 0)
              showTable.run(out);
           else
           {
              out.println("<CENTER><B>No more orders, Frank</CENTER></B>");
              showButtons = false;
           }
        }
        else
        {
           out.println("<P>Connection to server failed");
        }

        out.println("</TABLE>");



        if (showButtons)
        {
           out.println("<FORM>");
           out.println("<INPUT TYPE=SUBMIT VALUE=\"First Order\"  NAME=action>");
           out.println("<INPUT TYPE=SUBMIT VALUE=\"Print Mailing Label\"   NAME=action>");
           out.println("<INPUT TYPE=SUBMIT VALUE=\"Done With This Order\"  NAME=action>");
           out.println("<INPUT TYPE=SUBMIT VALUE=\"Next Order\" NAME=action>");
           out.println("</FORM>");
        }

        if (extraMessage.length() > 0)
           out.println(extraMessage);


        out.println("</BODY>");
        out.println("</HTML>");

        out.close();

        actionCode = NONE;
    }



    private int checkForOrder(byte[] ID)
    {
        int orderID = 0;

        try
        {
           KeyedDataQueueEntry dqData = dq.peek(ID, 1, "GT");

           if (dqData != null)
           {
              orderID = intConverter.toInt(dqData.getKey(), 0);
           }
        }
        catch (Exception e) { System.out.println("ProcesOrderServlet.checkForOrder"); e.printStackTrace(); }

        return orderID;
    }



    private boolean finishOrder(int orderID)
    {
        // Remove the item from the data queue
        try
        {
           KeyedDataQueueEntry dqData = dq.read(intConverter.toBytes(orderID));
        }
        catch (Exception e) { dumpDebugMessage("ProcesOrderServlet.done (remove for DQ failed)", e); }



        // Update the history log
        try
        {
           java.util.Date today = new java.util.Date();
           String temp = "\n" + " \n" + "** ORDER PROCESSED -- Date: " + today.toString() + "\n" + "   OrderID = " + orderID;

           IFSFileOutputStream log = new IFSFileOutputStream(system,
                                                          "/MyPop/OrderLog",
                                                          IFSFileOutputStream.SHARE_ALL,
                                                          true);
           log.write(temp.getBytes());
           log.flush();
           log.close();
        }
        catch (Exception e) { dumpDebugMessage("ProcesOrderServlet.done (write to log failed)", e); }



        // Update the units available in the database by subtracting
        // the number we just shipped.  Do this for each item
        // in the order.
        try
        {
           queryPS.setInt(1, orderID);

           ResultSet rs = queryPS.executeQuery();

           while (rs.next())
           {
              int amountLeft = rs.getInt("QUANTITYAVAILABLE") - rs.getInt("QUANTITYORDERED");
              int catalogID  = rs.getInt("CATALOGID");

              updatePS.setInt(1, amountLeft);
              updatePS.setInt(2, catalogID);

              updatePS.executeUpdate();
           }
        }
        catch (Exception e) { dumpDebugMessage("ProcesOrderServlet.done (update units available failed)", e); }

        return true;
    }





    private boolean printMailingLabel(int orderID, ServletOutputStream out)
    {

       try
       {
          queryPS2.setInt(1, orderID);

          ResultSet rs = queryPS2.executeQuery();

          if (rs.next())
          {
             int customerID = rs.getInt("CUSTOMERID");

             ProgramCall pc = new ProgramCall(system);

             ProgramParameter[] parmlist = new ProgramParameter[1];

             byte[] data = textConverter4.toBytes(new Integer(customerID).toString());
             parmlist[0] = new ProgramParameter(data);

             pc.setProgram("/QSYS.LIB/MYPOP.LIB/PRTLABEL.PGM", parmlist);

             if (pc.run()!=true)
             {
                AS400Message[] msgList = pc.getMessageList();

                dumpDebugMessage("ProcessOrderServlet.printMailingLabel (pgm call msg: )", null);
                dumpDebugMessage(msgList[0].getText(), null);
                out.println("<P>" + msgList[0].getText());

             }
          }
          else
          {
             dumpDebugMessage("ProcessOrderServlet.printMailingLabel (order query failed)", null);
             out.println("<P> rs.next() failed");
          }
       }
       catch (Exception e)
       {
          try { out.println("<P>" + e.getMessage()); } catch (Exception e2) {}
          dumpDebugMessage("ProcessOrderServlet.printMailingLabel", e);
       }
       return true;
    }



    void dumpDebugMessage(String message, Exception e)
    {
       if (debugOn)
       {
          System.out.println(message);

          if (e != null)
             e.printStackTrace();
       }
    }


}
