/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import com.ibm.as400.vaccess.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class SendMessage extends JComponent
{
    public static final int INITIAL_WIDTH = 550;
    public static final int INITIAL_HEIGHT = 250;

    boolean isJComponent_ = true;

    JFrame frame_;

    String toUser_      = null;
    String toUserLabel_ = null;

    AS400 system;

    JTextField user;
    JTextArea  msg;
    JCheckBox  checkBox;

    public SendMessage(AS400 sys, boolean isJComponent)
    {
        super();

        system = sys;

        isJComponent_ = isJComponent;

        if (isJComponent_)
            run();
    }


    public SendMessage(AS400 sys, String label, String toUser, boolean isJComponent)
    {
        super();

        system = sys;

        toUserLabel_ = label;
        toUser_      = toUser;

        isJComponent_ = isJComponent;

        if (isJComponent_)
            run();
    }


    public static void main (String[] args)
    {
        // If a system was not specified, then display help text and
        // exit.
        if (args.length == 0)
        {
            System.out.println("Usage: java SendMessage <system> <userid> <password> <specificUid>");
            return;
        }

        try
        {
           AS400 sys = new AS400(args[0]);

           if (args.length > 1)
              sys.setUserId(args[1]);

           if (args.length > 2)
              sys.setPassword(args[2]);

           SendMessage sendMessage = new SendMessage(sys, false);

           if (args.length > 3)
              sendMessage.setSpecificDestination("MyPop", args[3]);

           sendMessage.run();
        }
        catch (Exception e)
        {
           System.out.println("Failed in SendMessage (main)");
           e.printStackTrace();
        }
    }



    public void setSpecificDestination(String label, String uid)
    {
        toUserLabel_ = label;
        toUser_      = uid;
    }


    public void setFrame(JFrame frame)
    {
        frame_ = frame;
    }


    public void run()
    {
        try
        {
            if (!isJComponent_)
            {
                // Create a frame.
                frame_ = new JFrame ("Send a Message");

                // When the frame closes, exit.
                frame_.addWindowListener (new WindowAdapter ()
                {
                    public void windowClosing(WindowEvent e)
                    {
                        frame_.dispose();

                        if (!isJComponent_) {
                            System.exit(0);
                        }
                    }
                } );
            }

            // Create an error dialog adapter.  This will display
            // any errors to the user.
            ErrorDialogAdapter errorHandler = new ErrorDialogAdapter (frame_);

            JPanel userPanel = new JPanel();
            userPanel.setLayout (new FlowLayout(FlowLayout.LEFT));

            JLabel userLabel = new JLabel("To:");
            userPanel.add(userLabel);

            user = new JTextField(10);
            userPanel.add(user);

            if ((toUserLabel_ != null) && (toUserLabel_.length() > 0))
            {
               JLabel l = new JLabel("or ");
               checkBox = new JCheckBox(toUserLabel_);
               userPanel.add(l);
               userPanel.add(checkBox);
            }


            if (!isJComponent_) {
                frame_.getContentPane().add("North", userPanel);
            }
            else {
                add("North", userPanel);
            }


            JPanel textPanel = new JPanel();

            JLabel textLabel = new JLabel("Text:");
            textPanel.add(textLabel);

            msg  = new JTextArea(3, 50);
            textPanel.add(msg);

            if (!isJComponent_) {
                frame_.getContentPane().add("Center", textPanel);
            }
            else {
                add("North", textPanel);
            }


            // Create a button for displaying results in a table.
            JButton sendButton = new JButton("Send");
            sendButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   // System.out.println("send pressed");
                   // System.out.println(user.getText());
                   // System.out.println(msg.getText());

                   String popupText;

                   String target;

                   if ((checkBox != null) && (checkBox.isSelected()))
                      target = QSYSObjectPathName.toPath ("QUSRSYS", toUser_, "MSGQ");
                   else
                      target = QSYSObjectPathName.toPath("QUSRSYS", user.getText(), "MSGQ");

                   try
                   {
                      MessageQueue mq = new MessageQueue(system, target);
                      mq.sendInformational(msg.getText());
                      popupText = "Message Sent";
                   }
                   catch (Exception e) { System.out.println("e.getMessage()"); popupText = e.getMessage(); }

                   Popup p = new Popup(" ", popupText, " ", false);
                   p.run();

                   frame_.dispose();
                }
            } );

            // Create a button for displaying results in a form.
            JButton cancelButton = new JButton("Cancel");
            cancelButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                   // System.out.println("cancel pressed");
                   frame_.dispose();
                }
            } );

            // Add the buttons to the GUI.
            JPanel buttonPanel = new JPanel();
            buttonPanel.add(sendButton);
            buttonPanel.add(cancelButton);

            if (!isJComponent_) {
                frame_.getContentPane().add("South", buttonPanel);
                frame_.setSize(INITIAL_WIDTH, INITIAL_HEIGHT);
                Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
                frame_.setLocation(screenSize.width/2 - INITIAL_WIDTH/2,
                                   screenSize.height/2 - INITIAL_HEIGHT/2);
                frame_.setResizable(true);
                frame_.setVisible(true);
                frame_.pack ();
                frame_.show ();
            }
            else {
                add("South", buttonPanel);
            }
        }
        catch (Exception e)
        {
            System.out.println ("Error: " + e.getMessage ());
            if (!isJComponent_) {
                System.exit (0);
            }
        }
    }
}
