/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////
//
// This class is used when viewing the contents of a database by using
// Record Level Access.  The database file is interpreted as a
// Sequential File.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import java.sql.*;
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class ShowRecordsServlet extends Object
{
    private AS400  system;

    private String fileName;

    private boolean showHeader  = true;

    private int columnCount;

    private Hashtable columnName;
    private Hashtable omitColumn;

    RecordFormat format;
    SequentialFile file;

    boolean debugOn = true;

    private ShowRecordsServlet() { }


    public ShowRecordsServlet(AS400 sys, String file)
    {
        columnName = new Hashtable(100);
        omitColumn = new Hashtable(100);

        system   = sys;
        fileName = file;
    }



    public void setShowHeader(boolean value)
    {
       showHeader = value;
    }



    public void setColumnName(int columnNumber, String value)
    {
       try
       {
          columnName.remove(new Integer(columnNumber));
          columnName.put(new Integer(columnNumber), value);
       }
       catch (Exception e) { }
    }



    public void setIncludeColumn(int columnNumber, boolean value)
    {
       try
       {
          omitColumn.remove(new Integer(columnNumber));

          if (! value)
             omitColumn.put(new Integer(columnNumber), "1");
       }
       catch (Exception e) { }
    }



    public void setup()
    {
        try
        {
           AS400FileRecordDescription description = new AS400FileRecordDescription(system, fileName);
           format = description.retrieveRecordFormat()[0];
           columnCount = format.getNumberOfFields();
           file = new SequentialFile(system, fileName);
           file.setRecordFormat(format);
           file.open(SequentialFile.READ_ONLY, 1024, SequentialFile.COMMIT_LOCK_LEVEL_NONE);
        }
        catch (Exception e) { dumpDebugMessage("ShowRecordServlet.setup", e); }
    }



    public void run(ServletOutputStream out)
                    throws IOException
    {
        try
        {
           file.positionCursorToFirst();

              if (showHeader)
                 dumpEntry(out);

              dumpHeadings(out);

              Record r = file.readNext();

              while (r != null)
              {
                 dumpRow(r, out);
                 r = file.readNext();
              }

              if (showHeader)
                 dumpEnd(out);
        }
        catch (Exception e) { dumpDebugMessage("ShowRecordServlet.setup", e); }
    }



    private void dumpEntry(ServletOutputStream out)
                    throws IOException
    {
       out.println("<TABLE border=1>");
    }



    private void dumpEnd(ServletOutputStream out)
                    throws IOException
    {
       out.println("</TABLE>");
    }



    private void dumpHeadings(ServletOutputStream out)
                              throws IOException
    {
       out.println("<TR>");

       for (int i = 0; i < columnCount; i++)
       {
          if (! omitColumn.containsKey(new Integer(i)))
          {
             String columnLabel = (String) columnName.get(new Integer(i));

             if (columnLabel == null)
             {
                try
                {
                   columnLabel = format.getFieldDescription(i).getFieldName();
                }
                catch (Exception e) { System.out.println("error getting label"); }

                if (columnLabel == null)
                   columnLabel = " ";
             }

             out.println("<TD><B><CENTER>");

             out.println(columnLabel);

             out.println("</CENTER></B></TD>");
          }
       }

       out.println("</TR>");
    }



    private void dumpRow(Record r, ServletOutputStream out)
                         throws IOException
    {
       out.println("<TR>");

       for (int i = 0; i < columnCount; i++)
       {
          if (! omitColumn.containsKey(new Integer(i)))
          {

             String columnData = null;

             try
             {
                columnData = r.getField(i).toString();
             }
             catch (Exception e) { System.out.println("error getting string"); }

             if (columnData == null)
                columnData = "<BR>";

             out.println("<TD>");

             out.println(columnData);

             out.println("</TD>");
          }
       }

       out.println("</TR>");
    }



    void dumpDebugMessage(String message, Exception e)
    {
       if (debugOn)
       {
          System.out.println(message);

          if (e != null)
             e.printStackTrace();
       }
    }

}
