/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import java.sql.*;
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class ShowTableServlet extends Object
{
    private AS400  system;
    private String user;
    private String password;

    private String collectionName;
    private String query;

    private ResultSet rs;
    private ResultSetMetaData rsmd;
    private Statement select;
    private Connection connection;

    private boolean showHeader  = true;

    private int columnCount;

    private Hashtable columnName;
    private Hashtable omitColumn;

    private ShowTableServlet() { }

    public ShowTableServlet(AS400 sys)
    {
        columnName = new Hashtable(100);
        omitColumn = new Hashtable(100);

        system = sys;
    }

    public void setCollection(String collection)
    {
       collectionName = collection;
    }

    public void setQuery(String q)
    {
       query = q;
    }

    public void setShowHeader(boolean value)
    {
       showHeader = value;
    }



    public void setColumnName(int columnNumber, String value)
    {
       try
       {
          columnName.remove(new Integer(columnNumber));
          columnName.put(new Integer(columnNumber), value);
       }
       catch (Exception e) { }
    }



    public void setIncludeColumn(int columnNumber, boolean value)
    {
       try
       {
          omitColumn.remove(new Integer(columnNumber));

          if (! value)
             omitColumn.put(new Integer(columnNumber), "1");
       }
       catch (Exception e) { }
    }


    public void setup()
    {
        try
        {
            DriverManager.registerDriver(new AS400JDBCDriver());
            connection = DriverManager.getConnection("jdbc:as400://" + system.getSystemName() + "/" + collectionName + "; user=" + system.getUserId());
            select = connection.createStatement();
        }
        catch (Exception e)
        {
            System.out.println ("Error in Setup");
            e.printStackTrace();
        }
    }


    public void run(ServletOutputStream out)
                    throws IOException
    {
        try
        {
           rs   = select.executeQuery(query);
           rsmd = rs.getMetaData();
           columnCount = rsmd.getColumnCount();

           if (showHeader)
              dumpEntry(out);

           dumpHeadings(out);

           while (rs.next())
              dumpRow(out);

           if (showHeader)
              dumpEnd(out);
        }
        catch (Exception e)
        {
            System.out.println ("Error in Customer Database 2");
            e.printStackTrace();
        }
    }


    private void dumpEntry(ServletOutputStream out)
                    throws IOException
    {
       out.println("<TABLE border=1>");
    }

    private void dumpEnd(ServletOutputStream out)
                    throws IOException
    {
       out.println("</TABLE>");
    }



    private void dumpHeadings(ServletOutputStream out)
                    throws IOException
    {
       out.println("<TR>");

       for (int i = 1; i <= columnCount; i++)
       {
          if (! omitColumn.containsKey(new Integer(i)))
          {
             String columnLabel = (String) columnName.get(new Integer(i));

             if (columnLabel == null)
             {
                try
                {
                   columnLabel = rsmd.getColumnLabel(i);
                }
                catch (Exception e) { System.out.println("error getting label"); }

                if (columnLabel == null)
                   columnLabel = " ";
             }

             out.println("<TD><B><CENTER>");

             out.println(columnLabel);

             out.println("</CENTER></B></TD>");
          }
       }

       out.println("</TR>");
    }



    private void dumpRow(ServletOutputStream out)
                         throws IOException
    {
       out.println("<TR>");

       for (int i = 1; i <= columnCount; i++)
       {
          if (! omitColumn.containsKey(new Integer(i)))
          {

             String columnData = null;

             try
             {
                columnData = rs.getString(i);
             }
             catch (Exception e) { System.out.println("error getting string"); }

             if (columnData == null)
                columnData = " ";

             out.println("<TD>");

             out.println(columnData);

             out.println("</TD>");
          }
       }

       out.println("</TR>");
    }
}
