/////////////////////////////////////////////////////////////////////////
//
// Warehouse example.  This program accesses various resources on the
// AS/400.
//
// Command syntax:
//    java mypop.Warehouse
//
/////////////////////////////////////////////////////////////////////////
//
// This source shows an example use of the AS/400 Toolbox for Java.
// IBM grants you a nonexclusive license to use this as an example
// from which you can generate similar function tailored to
// your own specific needs.
//
// This sample code is provided by IBM for illustrative purposes
// only. These examples have not been thoroughly tested under all
// conditions. IBM, therefore, cannot guarantee or imply
// reliability, serviceability, or function of these programs.
//
// All programs contained herein are provided to you "AS IS"
// without any warranties of any kind.  The implied warranties of
// merchantablility and fitness for a particular purpose are
// expressly disclaimed.
//
// AS/400 Toolbox for Java
// (C) Copyright IBM Corp. 1997, 1999
// All rights reserved.
// US Government Users Restricted Rights -
// Use, duplication, or disclosure restricted
// by GSA ADP Schedule Contract with IBM Corp.
//
/////////////////////////////////////////////////////////////////////////

package mypop;

import com.ibm.as400.access.*;
import com.ibm.as400.vaccess.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.beans.*;
import java.io.*;
import java.sql.*;
import java.util.*;
import java.net.*;

public class Warehouse extends JApplet
{
    public static final int TOTAL_PANELS = 8;
    public static final int WIDTH = 750;
    public static final int HEIGHT = 550;
    public static final int INITIAL_WIDTH = 400;
    public static final int INITIAL_HEIGHT = 200;

    String sys;
    String uid;
    String pwd;

    boolean isApplet_ = true;
    boolean appletStarted = false;
    URL thisURL_;

    Warehouse warehouse;

    Dialog aboutBox_;
    JTabbedPane warehousePane_;
    JFrame frame_ = null;
    JMenuItem windowsMenuItem_;
    JMenuItem metalMenuItem_;

    SQLConnection conn_;
    Cursor workingCursor_ = new Cursor(Cursor.WAIT_CURSOR);
    Cursor defaultCursor_ = new Cursor(Cursor.DEFAULT_CURSOR);





    public static void main (String[] args)
    {
        Warehouse w = new Warehouse(false);
        w.Main(args);
    }

    public void Main (String[] args)
    {
        warehouse = new Warehouse(false);

        if (setup())
        {
           warehouse.run();
        }
    }





    // Needed for applet loading from a browser or appletviewer.
    public Warehouse() { }


    public Warehouse(boolean isApplet)
    {
        isApplet_ = isApplet;
    }




    void setSystem(String system)
    {
       sys = system;
    }

    void setUserid(String userid)
    {
       uid = userid;
    }

    void setPassword(String p)
    {
       pwd = p;
    }

    public void setURL(URL thisURL)
    {
        thisURL_ = thisURL;
    }




    private boolean setup()
    {
       if (warehouse != null)
       {
          try
          {
             ResourceBundle rb = ResourceBundle.getBundle("mypop.mypop");

             String system = rb.getString("SYSTEM");

             if (system != null)
             {
                warehouse.setSystem(system);
             }
             else
             {
                System.out.println("Setup failed, could not get AS/400 name out of mypop.properties");
                return false;
             }

             String password = rb.getString("FRANKPWD");

             if (password != null)
             {
                warehouse.setPassword(password);
             }
             else
             {
                System.out.println("Setup failed, could not get Frank's password out of mypop.properties");
                return false;
             }

             warehouse.setUserid("FRANK");

             return true;

          }
          catch (Exception e)
          {
             System.out.println("Failed, could not find mypop.properties");
             e.printStackTrace();
             return false;
          }
       }
       else
       {
          System.out.println("Failed, warehouse object is null");
          return false;
       }
    }




    public void init()
    {
        thisURL_ = this.getCodeBase();
        System.out.println("this url = " + thisURL_);
        warehouse = new Warehouse(true);
        warehouse.setup();
        warehouse.setSystem(thisURL_.getHost());
        warehouse.setURL(thisURL_);
    }


    public void start()
    {
        if (!appletStarted)
        {
           warehouse.run();
           appletStarted = true;
        }
    }



    public void run()
    {
        try
        {
            // Create a frame.
            frame_ = new JFrame ();

            // Load the components.
            loadComponents();
        }
        catch (Exception e)
        {
           System.out.println("Exception in Warehouse.run");
           e.printStackTrace();

           if (!isApplet_)
           {
               System.exit(0);
           }
        }
    }









    private void loadComponents()
    {
        try
        {
            AS400 system = new AS400(sys);
            system.setUserId(uid);
            system.setPassword(pwd);



            IFSFile f0 = new IFSFile(system, "/QSYS.LIB/FRANK.USRPRF");
            if (!f0.exists())
               System.out.println("Warning - Frank's userid does not exist");

            IFSFile f00 = new IFSFile(system, "/QSYS.LIB/MYPOP.USRPRF");
            if (!f00.exists())
               System.out.println("Warning - MyPop's userid does not exist");



            frame_.setTitle("MyPop Warehouse");

            JPanel progressPanel = new JPanel()
            {
                public Insets getInsets()
                {
                   return new Insets(20,30,20,30);
                }
            };

            progressPanel.setLayout(new BoxLayout(progressPanel, BoxLayout.Y_AXIS));
            frame_.getContentPane().add(progressPanel, BorderLayout.CENTER);

            Dimension d = new Dimension(400, 20);
            JLabel progressLabel = new JLabel("Loading, please wait...");
            progressLabel.setAlignmentX(CENTER_ALIGNMENT);
            progressLabel.setMaximumSize(d);
            progressLabel.setPreferredSize(d);
            progressPanel.add(progressLabel);
            progressPanel.add(Box.createRigidArea(new Dimension(1,20)));

            JProgressBar progressBar = new JProgressBar();
            progressLabel.setLabelFor(progressBar);
            progressBar.setAlignmentX(CENTER_ALIGNMENT);
            progressBar.setMinimum(0);
            progressBar.setMaximum(Warehouse.TOTAL_PANELS);
            progressBar.setValue(0);
            progressPanel.add(progressBar);

            // show the frame
            frame_.setSize(INITIAL_WIDTH, INITIAL_HEIGHT);
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            frame_.setLocation(screenSize.width/2 - INITIAL_WIDTH/2,
                              screenSize.height/2 - INITIAL_HEIGHT/2);
            frame_.show();
            frame_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

            JTabbedPane warehousePane_ = new JTabbedPane();
            int currentProgress = 0;



            progressLabel.setText("Loading the message queue panel");

            IFSFile f2 = new IFSFile(system, "/QSYS.LIB/QUSRSYS.LIB/FRANK.MSGQ");
            if (!f2.exists())
               System.out.println("Warning - Frank's message queue does not exist");

            IFSFile f3 = new IFSFile(system, "/QSYS.LIB/QUSRSYS.LIB/MYPOP.MSGQ");
            if (!f3.exists())
               System.out.println("Warning - MyPop's message queue does not exist");

            MsgQueue mq = new MsgQueue(system, "MyPop", "MYPOP", true);
            mq.setFrame(frame_);
            warehousePane_.addTab("Message Queue", mq);
            progressBar.setValue(++currentProgress);





            progressLabel.setText("Loading the history log panel");

            IFSFile f1 = new IFSFile(system, "/mypop/OrderLog");
            if (!f1.exists())
               System.out.println("Warning - Order log does not exist");

            HistoryLog historyLog = new HistoryLog(system, true);
            historyLog.setFrame(frame_);
            warehousePane_.addTab("History Log", historyLog);
            progressBar.setValue(++currentProgress);






            system.addPasswordCacheEntry(sys, uid, pwd);

            progressLabel.setText("Loading the process order panel");

            IFSFile f4 = new IFSFile(system, "/QSYS.LIB/MYPOP.LIB/ORDERS.FILE");
            if (!f4.exists())
               System.out.println("Warning - Orders database does not exist");

            IFSFile f5 = new IFSFile(system, "/QSYS.LIB/MYPOP.LIB/PRODCAT.FILE");
            if (!f5.exists())
               System.out.println("Warning - Catalog database does not exist");

            ProcessOrder po = new ProcessOrder(system, true);
            po.setFrame(frame_);
            po.run();
            warehousePane_.addTab("Process Orders", po);
            progressBar.setValue(++currentProgress);

            progressLabel.setText("Loading the Orders Database Panel");
            DBOrders order = new DBOrders(sys, uid, pwd, true);
            order.setFrame(frame_);
            order.run();
            warehousePane_.addTab("Orders Database",order);
            progressBar.setValue(++currentProgress);

            progressLabel.setText("Loading the Customer Database Panel");
            CustomerDB cust = new CustomerDB(system, true);
            cust.setFrame(frame_);
            cust.run();
            warehousePane_.addTab("Customer Database", cust);
            progressBar.setValue(++currentProgress);

            progressLabel.setText("Loading the Product Catalog Database Panel");
            CatalogDB catalog = new CatalogDB(system, true);
            catalog.setFrame(frame_);
            catalog.run();
            warehousePane_.addTab("Product Catalog", catalog);
            progressBar.setValue(++currentProgress);



            progressLabel.setText("Set up the data queue reader");

            IFSFile f6 = new IFSFile(system, "/QSYS.LIB/MYPOP.LIB/WAREHOUSE.DTAQ");
            if (!f6.exists())
               System.out.println("Warning - Warehouse data queue does not exist");

            DQAlert dq = new DQAlert(system, true);
            dq.start();
            progressBar.setValue(++currentProgress);





            System.out.println("\n\n\n");



            frame_.getContentPane().removeAll();
            frame_.getContentPane().setLayout(new BorderLayout());
            frame_.getContentPane().add(createMenuBar(), BorderLayout.NORTH);
            frame_.getContentPane().add(warehousePane_, BorderLayout.CENTER);
            frame_.setLocation(screenSize.width/2 - WIDTH/2,
                               screenSize.height/2 - HEIGHT/2);
            frame_.setSize(WIDTH, HEIGHT);
            frame_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
            frame_.setTitle("MyPop Warehouse");
            frame_.validate();
            frame_.repaint();

            // When the frame closes, exit.
            frame_.addWindowListener (new WindowAdapter ()
            {
                public void windowClosing(WindowEvent e)
                {
                    frame_.dispose();
                    if (!isApplet_) {
                        System.exit(0);
                    }
                }
            } );
        }
        catch (Exception e)
        {
            e.printStackTrace();
            System.out.println ("Error: " + e.getMessage ());
            if (!isApplet_) {
                System.exit (0);
            }
        }
    }


    private JMenuBar createMenuBar()
    {
        JMenuBar menuBar = new JMenuBar();

        JMenuItem mi;

        // File Menu
        JMenu file = (JMenu) menuBar.add(new JMenu("File"));
        file.setMnemonic('F');
        mi = (JMenuItem) file.add(new JMenuItem("About"));
        mi.setMnemonic('t');
        mi.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                Popup p = new Popup(" ", "MyPop Warehouse Version .4", " ", false);
                p.run();
            }
        });

        file.add(new JSeparator());
        mi = (JMenuItem) file.add(new JMenuItem("Exit"));
        mi.setMnemonic('x');
        mi.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                frame_.dispose();
                if (!isApplet_) {
                    System.exit(0);
                }
            }
        });


        // Options Menu
        JMenu options = (JMenu) menuBar.add(new JMenu("Options"));
        options.setMnemonic('p');

        // Look and Feel Radio control
        ButtonGroup group = new ButtonGroup();
        ToggleUIListener toggleUIListener = new ToggleUIListener();

        // Windows UI
        windowsMenuItem_ = (JRadioButtonMenuItem) options.add(new JRadioButtonMenuItem("Windows Style Look and Feel"));
        windowsMenuItem_.setSelected(UIManager.getLookAndFeel().getName().equals("Windows"));
        group.add(windowsMenuItem_);
        windowsMenuItem_.addItemListener(toggleUIListener);
        windowsMenuItem_.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.ALT_MASK));

        // Metal UI
        metalMenuItem_ = (JRadioButtonMenuItem) options.add(new JRadioButtonMenuItem("Metal Look and Feel"));
        metalMenuItem_.setSelected(UIManager.getLookAndFeel().getName().equals("Metal"));
        metalMenuItem_.setSelected(true);
        group.add(metalMenuItem_);
        metalMenuItem_.addItemListener(toggleUIListener);
        metalMenuItem_.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, ActionEvent.ALT_MASK));

        return menuBar;
    }


    private ImageIcon loadImageIcon(String fileName, String description) {
        if(!isApplet_) {
            return new ImageIcon(fileName, description);
        }
        else {
            URL url;
            try {
                System.out.println("this url = " + thisURL_);
                url = new URL(thisURL_, fileName);
            }
            catch(Exception e) {
                e.printStackTrace();
                System.err.println("Error trying to load image " + fileName);
                return null;
            }
            return new ImageIcon(url, description);
        }
    }


    /**
     * Switch the user interface between the Windows and the Metal Look and Feel
     */
    class ToggleUIListener implements ItemListener {
     public void itemStateChanged(ItemEvent e) {
         frame_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
         JRadioButtonMenuItem rb = (JRadioButtonMenuItem) e.getSource();
            try {
                if(rb.isSelected() && rb.getText().equals("Windows Style Look and Feel")) {
                    UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
                    SwingUtilities.updateComponentTreeUI(frame_);
                }
                else if(rb.isSelected() && rb.getText().equals("Metal Look and Feel")) {
                    UIManager.setLookAndFeel("com.sun.java.swing.plaf.metal.MetalLookAndFeel");
                    SwingUtilities.updateComponentTreeUI(frame_);
                }
            }
            catch (UnsupportedLookAndFeelException exc) {
                // Error - unsupported L&F
                rb.setEnabled(false);
                System.err.println("Unsupported LookAndFeel: " + rb.getText());

                // Set L&F to Metal
                try {
                    metalMenuItem_.setSelected(true);
                    UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
                    SwingUtilities.updateComponentTreeUI(frame_);
                }
                catch (Exception exc2) {
                    System.err.println("Could not load LookAndFeel: " + exc2);
                }
            }
            catch (Exception exc) {
                rb.setEnabled(false);
                System.err.println("Could not load LookAndFeel: " + rb.getText());
            }

            frame_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
     }

    }


}
